"""
Copyright (c) 2024 by FlashInfer team.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import torch

try:
    from . import _kernels
except ImportError as e:
    import os
    import logging

    if os.environ.get("BUILD_DOC", "0") == "1":
        _kernels = None
        logging.warning("Kernels are not loaded in documentation build mode.")
    else:
        raise e


def rmsnorm(x: torch.Tensor, w: torch.Tensor, eps: float = 1e-6):
    r"""Root mean square normalization.

    Parameters
    ----------
    x: torch.Tensor
        Input tensor, shape (batch_size, hidden_size).
    w: torch.Tensor
        Weight tensor, shape (hidden_size,).
    eps: float
        Epsilon for numerical stability.

    Returns
    -------
    y: torch.Tensor
        Normalized tensor, shape (batch_size, hidden_size).
    """
    return _kernels.rmsnorm(x, w, eps)
